<?php

namespace Drupal\ultimenu\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\ultimenu\UltimenuManagerInterface;
use Drupal\ultimenu\Ajax\UltimenuHtmlCommand;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Provides controller for Ultimenu region route.
 */
class UltimenuController extends ControllerBase {

  /**
   * The context handler.
   *
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * The context repository interface.
   *
   * @var \Drupal\Core\Plugin\Context\ContextRepositoryInterface
   */
  protected $contextRepository;

  /**
   * The Ultimenu manager service.
   *
   * @var \Drupal\ultimenu\UltimenuManagerInterface
   */
  protected $ultimenuManager;

  /**
   * Constructs a new DefaultController object.
   */
  public function __construct(ContextHandlerInterface $context_handler, ContextRepositoryInterface $context_repository, UltimenuManagerInterface $ultimenu_manager) {
    $this->contextHandler = $context_handler;
    $this->contextRepository = $context_repository;
    $this->ultimenuManager = $ultimenu_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('context.handler'),
      $container->get('context.repository'),
      $container->get('ultimenu.manager')
    );
  }

  /**
   * Loads and renders a region via AJAX.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return array
   *   Return the requested region based on the given region parameters.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *
   * @see http://symfony.com/doc/current/routing.html#required-and-optional-placeholders
   */
  public function load(Request $request) {
    $cid = $request->query->get('cid');
    $cur = $request->query->get('cur');
    $sub = $request->query->get('sub');

    if ($cid && $cur) {
      list($menu_name, $menu_item_id, $title) = array_pad(array_map('trim', explode("|", rawurldecode($cid), 3)), 3, NULL);
      $plugin_id = 'ultimenu_block:ultimenu-' . $menu_name;

      if ($regions = $this->ultimenuManager->getSetting('regions')) {
        $rid = $this->ultimenuManager->getTool()->getRegionKey($menu_name, $menu_item_id, $title);

        if (!empty($regions[$rid])) {
          $block = $this->ultimenuManager->getEntityTypeManager()->getStorage('block')->loadByProperties(['plugin' => $plugin_id]);
          $block = reset($block);
          $config = $block->get('settings');
          $config['current_path'] = $cur;
          $config['has_submenu'] = $sub;
          $config['menu_name'] = $menu_name;
          $config['title'] = $title;
          $config['uuid'] = $menu_item_id;

          // Creates AJAX response.
          $response = new AjaxResponse();

          // Creates render context.
          $context = new RenderContext();

          // Builds the flyout.
          $render = $this->ultimenuManager->getRenderer()->executeInRenderContext($context, function () use ($rid, $config) {
            return $this->ultimenuManager->buildFlyout($rid, $config);
          });

          // Prevents empty render from screwing up the response. Hence the
          // region is provided, but none of the blocks is given:
          // The render array has not yet been rendered, hence
          // not all attachments have been collected yet.
          if ($render) {
            // Add metadata.
            if (!$context->isEmpty()) {
              $bubbleable_metadata = $context->pop();
              BubbleableMetadata::createFromRenderArray($render)
                ->merge($bubbleable_metadata)
                ->applyTo($render);
            }

            $response->addCommand(new UltimenuHtmlCommand('[data-ultiajax-region="' . $rid . '"]', $render, NULL, 'region'));
          }
          return $response;
        }
      }
      throw new AccessDeniedHttpException();
    }

    throw new NotFoundHttpException();
  }

}
